@@ -93,7 +93,7 @@ rclaytonCopula <- function(n, copula) {
 
 pclaytonCopula <- function(copula, u) {
   dim <- copula@dimension
-  if(!is.matrix(u)) u <- matrix(u, ncol = dim)
+  stopifnot(!is.null(d <- ncol(u)), dim == d)
   alpha <- copula@parameters[1]
   if (abs(alpha) <= .Machine$double.eps^.9) return (apply(u, 1, prod))
   cdf <- copula@exprdist$cdf
@@ -105,13 +105,13 @@ pclaytonCopula <- function(copula, u) {
 ## Nowhere used (!)
 dclaytonCopula <- function(copula, u, ...) {
   dim <- copula@dimension
-  if(!is.matrix(u)) u <- matrix(u, ncol = dim)
+  ## if(!is.matrix(u)) u <- matrix(u, ncol = dim)
   alpha <- copula@parameters[1]
   if (abs(alpha) <= .Machine$double.eps^.9) return (rep(1, nrow(u)))
   for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   val <- c(eval(copula@exprdist$pdf))
-  val[apply(u, 1, function(v) any(v < 0))] <- 0
-  val[apply(u, 1, function(v) any(v > 1))] <- 0
+  ## val[apply(u, 1, function(v) any(v < 0))] <- 0
+  ## val[apply(u, 1, function(v) any(v > 1))] <- 0
 ##   if (alpha < 0) {
 ##     cdf <- pCopula(u, copula)
 ##     bad <- cdf == 0
@@ -123,17 +123,18 @@ dclaytonCopula <- function(copula, u, ...) {
 dclaytonCopula.pdf <- function(u, copula, log=FALSE) {
   dim <- copula@dimension
   if (dim > 10) stop("Clayton copula PDF not implemented for dimension > 10.")
-  if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
+  ## if(!is.matrix(u)) u <- rbind(u, deparse.level = 0L)
+  stopifnot(!is.null(d <- ncol(u)), dim == d)
   for (i in 1:dim) assign(paste("u", i, sep=""), u[,i])
   alpha <- copula@parameters[1]
   if (abs(alpha) <= .Machine$double.eps^.9)
     return(rep.int(if(log) 0 else 1, nrow(u)))
   val <- pmax(c(eval(claytonCopula.pdf.algr[dim])),0)
-  ## clean up
-  val[apply(u, 1, function(v) any(v < 0))] <- 0
-  val[apply(u, 1, function(v) any(v > 1))] <- 0
+  ## clean up -- now happens in dCopula()
+  ## val[apply(u, 1, function(v) any(v < 0))] <- 0
+  ## val[apply(u, 1, function(v) any(v > 1))] <- 0
 
-  val[apply(u, 1, function(v) any(v == 0) & any(v > 0))] <- 0
+  ## val[apply(u, 1, function(v) any(v == 0) & any(v > 0))] <- 0
 
   ## if (alpha > 0)
   ## else
@@ -204,7 +205,7 @@ dTauClaytonCopula <- function(copula) {
 }
 
 pMatClayton <- function (u, copula, ...) {
-    stopifnot(ncol(u) == (d <- copula@dimension))
+    stopifnot(!is.null(d <- ncol(u)), d == copula@dimension)
     th <- copula@parameters
     if(d == 2 && !copClayton@paraConstr(th)) # for now, .. to support negative tau
         pclaytonCopula(copula, u=u)
@@ -213,7 +214,7 @@ pMatClayton <- function (u, copula, ...) {
 }
 
 dMatClayton <- function (u, copula, log = FALSE, ...) {
-    stopifnot(ncol(u) == (d <- copula@dimension))
+    stopifnot(!is.null(d <- ncol(u)), d == copula@dimension)
     th <- copula@parameters
     if(d == 2 && !copClayton@paraConstr(th)) # for now, .. to support negative tau
         dclaytonCopula.pdf(u, copula, log=log)
@@ -230,9 +231,8 @@ setMethod("pCopula", signature("numeric", "claytonCopula"),
 setMethod("pCopula", signature("matrix", "claytonCopula"), pMatClayton)
 
 setMethod("dCopula", signature("numeric", "claytonCopula"),
-	  function (u, copula, ...)
-	  ## was  dclaytonCopula.pdf
-          dMatClayton(matrix(u, ncol = dim(copula)), copula, ...))
+	  function (u, copula, log=FALSE, ...)
+	  dMatClayton(matrix(u, ncol = dim(copula)), copula, log=log, ...))
 setMethod("dCopula", signature("matrix", "claytonCopula"), dMatClayton)
 
 
